﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using static DesktopReader.Constants;

namespace DesktopReaderSample
{
    public partial class frmMain : Form
    {
        #region "Variables"
        //Create reader
        DesktopReader.Reader USBReader;
        List<ComboBox> listcboxKey = new List<ComboBox>();
        ComboBox[] arrcboxWriteData = new ComboBox[16]; 
        List<Byte[]> listEntolledTemplates = new List<Byte[]>();
        #endregion
        #region "Form Load/Close"
        public frmMain()
        {
            InitializeComponent();
        }
        private void frmMain_Load(object sender, EventArgs e)
        {
            subClearText();
            int X, Y;
            for (X = 0; X < 40; X++)
            {
                cboxSector.Items.Add(X.ToString());
            }
            cboxSector.Text = "0";
            for (X = 0; X < 16; X++)
            {
                cboxBlock.Items.Add(X.ToString());
            }
            cboxBlock.Text = "0";
            listcboxKey.Add(cboxKey0);
            listcboxKey.Add(cboxKey1);
            listcboxKey.Add(cboxKey2);
            listcboxKey.Add(cboxKey3);
            listcboxKey.Add(cboxKey4);
            listcboxKey.Add(cboxKey5);
            foreach (ComboBox cbox in listcboxKey)
            {
                for (X = 0; X < 256; X++)
                {
                    cbox.Items.Add(X.ToString());
                }
                cbox.Text = "255";
            }
            for (X = 0; X < 16; X++)
            {
                arrcboxWriteData[X] = new ComboBox();
                arrcboxWriteData[X].Width = 45;
                arrcboxWriteData[X].DropDownStyle = ComboBoxStyle.DropDownList;
                flpBlockDataWrite.Controls.Add(arrcboxWriteData[X]);
                arrcboxWriteData[X].Visible = true;
                for (Y = 0; Y < 256; Y++)
                {
                    arrcboxWriteData[X].Items.Add(Y.ToString());
                }
                arrcboxWriteData[X].Text = "0";
            }
            comboSecurityLevel.Text = "4";
            comboTemplateToReadFromCard.Text = "1";
            comboFingerToWriteInCard.Text = "1";
            comboFingerToDeleteFromCard.Text = "1";
            comboTimeout.Text = "3000";
            //Initialise reader
            USBReader = new DesktopReader.Reader();
            //Add event of reader connected/disconnected
            USBReader.USBEvent += USBEventHandler;
            //As at initialisation of the dll card reader may be already connected to PC, check for AttachedDevice property
            Text = "USB reader dll demo: " + Application.ProductVersion + " Desktopr reader Dll version: " + USBReader.GetType().Assembly.GetName().Version;
            EnableControl(USBReader.AttachedDevice);
            //Display reader firmware version
            lblVersion.Text = USBReader.AttachedDeviceVersion;
        }
        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            //Remove event of reader connected/disconnected
            USBReader.USBEvent -= USBEventHandler;
            USBReader.Dispose();
            USBReader = null;
        }
        #endregion
        #region "Form comands"
        private void btnGetCardID_Click(object sender, EventArgs e)
        {
            btnGetCardID.Enabled = false;
            try
            {
                subClearText();
                byte CardType = (byte)CardTypes.EM4100;
                if (rbMifare.Checked) { CardType = (byte)CardTypes.MIFARE; }
                if (rbHID.Checked) { CardType = (byte)CardTypes.HID; }
                if (rbAWID.Checked) { CardType = (byte)CardTypes.AWID; }
                //Ask reader for card number
                ReadIDResault Resault = USBReader.ReadCardID(CardType);
                //Display returned Error code
                lblErrorCode.Text = Enum.GetName(typeof(ErrorCodes), Resault.ErrorCode);
                //Check if operation has no error and display number
                if (Resault.ErrorCode == (byte)ErrorCodes.OK)
                {
                    {
                        lblCardType.Text = Enum.GetName(typeof(CardTypes), Resault.CardType);
                    }
                    //Display Mifare card type
                    if (Resault.CardType == (byte)CardTypes.MIFARE)
                    {
                        lblMifareType.Text = Enum.GetName(typeof(MifareTypes), Resault.MifareType);
                    }
                    //Display card ID byte array
                    lblID.Text = string.Join(", ", Array.ConvertAll(Resault.ID, Convert.ToString));
                    //Display full card ID as nuber
                    decimal decID = ByteToDecimal(Resault.ID);
                    lblID_Decimal.Text = decID.ToString();
                    //Display 7 byte UID converted to 4 byte NUI (only for Mifare 7 byte cards
                    lblIDConverted.Text = string.Join(", ", Array.ConvertAll(Resault.IDConverted7to4, Convert.ToString));
                    //Display converted 4 byte ID as single number
                    lblID_Converted_Decimal.Text = BitConverter.ToUInt32(Resault.IDConverted7to4, 0).ToString();
                    //Display ID length (bits)
                    lblIDSize.Text = Resault.UIDLength.ToString();
                }
            }
            catch { }
            btnGetCardID.Enabled = true;
        }
        private void btnReadBlock_Click(object sender, EventArgs e)
        {
            TableLayoutPanel2.Enabled = false;
            subClearText();
            //Select key type for sector access
            byte KeyType = (byte)MifareKeys.KeyA;
            if (rbB.Checked) { KeyType = (byte)MifareKeys.KeyB; }
            //Assemble key array
            byte[] key = new byte[6];
            for (int X = 0; X < 6; X++)
            {
                key[X] = Convert.ToByte(listcboxKey[X].Text);
            }
            //Read block
            ReadBlockResault Resault = USBReader.ReadMifareClassicBlock(KeyType, key, Convert.ToByte(cboxSector.Text), Convert.ToByte(cboxBlock.Text));
            //Display returned Error code
            lblReadBlockResponse.Text = Enum.GetName(typeof(ErrorCodes), Resault.ErrorCode);
            //Check if operation has no error and display block content
            if (Resault.ErrorCode == (byte)ErrorCodes.OK)
            {
                txtClassicData.Text = string.Join(", ",Array.ConvertAll(Resault.BlockData,Convert.ToString));
            }
            TableLayoutPanel2.Enabled = true;
        }
        private void btnWriteBlock_Click(object sender, EventArgs e)
        {
            btnReadBlock.Enabled = false;
            btnWriteBlock.Enabled = false;
            subClearText();
            //Select key type for sector access
            Byte KeyType = (byte)MifareKeys.KeyA;
            if (rbB.Checked) { KeyType = (byte)MifareKeys.KeyB; }
            //Assemble key array
            byte[] key = new byte[6];
            for (int X = 0; X < 6; X++)
            {
                key[X] = Convert.ToByte(listcboxKey[X].Text);
            }
            //Assemble block content
            byte[] Data = new byte[16];
            for (int X = 0; X < 16; X++)
            {
                Data[X] = Convert.ToByte(arrcboxWriteData[X].Text);
            }
            //Write data to card
            int Resault = USBReader.WriteMifareClassicBlock(KeyType, key, Convert.ToByte(cboxSector.Text), Convert.ToByte(cboxBlock.Text), Data);
            //Display operation resault
            lblReadBlockResponse.Text = Enum.GetName(typeof(ErrorCodes), Resault);
            btnReadBlock.Enabled = true;
            btnWriteBlock.Enabled = true;
        }
        private void btnGetFinger_Click(object sender, EventArgs e)
        {
            tlpBiometry.Enabled = false;
            try
            {
                subClearText();
                txtTemplateArray.Clear();
                //Read template from card
                FingerData Resault = USBReader.GetFingerTemlate();
                //Display returned Error code
                lblBiometryResponse.Text = Enum.GetName(typeof(ErrorCodes), Resault.ErrorCode);
                //If there is no error
                if (Resault.ErrorCode == (byte)ErrorCodes.OK)
                {
                    //display template byte array in text box
                    txtTemplateArray.Text = string.Join(", ",Array.ConvertAll(Resault.Template, Convert.ToString));
                    listEntolledTemplates.Add(Resault.Template);
                    lblEnrolledFingers.Text = listEntolledTemplates.Count.ToString();
                    comboWriteTemplateToCard.Items.Add(lblEnrolledFingers.Text);
                    comboWriteTemplateToCard.Text = lblEnrolledFingers.Text;
                    btnWriteFinger.Enabled = true;
                    comboWriteTemplateToCard.Enabled = true;
                    //Check is it Suprema (BIOPROX-USB) or DP (B100PROX-USB) template
                    if (Resault.TemplateType == (byte)FingerTemplateTypes.Suprema)
                    {
                        lblTemplateType.Text = "Suprema";
                        lblQuality.Text = Resault.Quality.ToString() + " %";
                        comboFingerToVerify.Enabled = true;
                        comboFingerToVerify.Items.Add(lblEnrolledFingers.Text);
                        comboFingerToVerify.Text = lblEnrolledFingers.Text;
                        btnVerify.Enabled = true;
                        btnFindMatch.Enabled = true;
                    }
                    else
                    {
                        lblTemplateType.Text = "DP";
                    }
                }
            }
            catch { }
            tlpBiometry.Enabled = true;
        }
        private void btnWriteFinger_Click(object sender, EventArgs e)
        {
            tlpBiometry.Enabled = false;
            try
            {
                subClearText();
                //Write template to card
                int response = USBReader.WriteFingerTemplateToCard(Convert.ToByte(comboFingerToWriteInCard.Text), listEntolledTemplates[Convert.ToInt32(comboWriteTemplateToCard.Text)-1]);
                //Display operation resault
                lblBiometryResponse.Text = Enum.GetName(typeof(ErrorCodes), response);
            }
            catch (Exception ex)
           {
                MessageBox.Show(ex.Message);
            }
        tlpBiometry.Enabled = true;
        }
        private void btnReadFingerFromCard_Click(object sender, EventArgs e)
        {
            tlpBiometry.Enabled = false;
            try
            {
                subClearText();
            lblBiometryResponse.Text = "";
            //Read finger from card
            FingerData Resault   = USBReader.ReadFingerFromCard(Convert.ToByte(comboTemplateToReadFromCard.Text));
                //Display operation resault
                lblBiometryResponse.Text = Enum.GetName(typeof(ErrorCodes), Resault.ErrorCode);
            //If OK, display template in textbox
            if (Resault.ErrorCode == (byte)ErrorCodes.OK)
                {
                    txtTemplateArray.Text = string.Join(", ", Array.ConvertAll(Resault.Template,Convert.ToString));
                }
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        tlpBiometry.Enabled = true;
        }
        private void btnDeleteFingerData_Click(object sender, EventArgs e)
        {
            tlpBiometry.Enabled = false;
            subClearText();
            //Delete finger from card
            int response = USBReader.DeleteFingerData(Convert.ToByte(comboFingerToDeleteFromCard.Text));
            //Display returned Error code
            lblBiometryResponse.Text = Enum.GetName(typeof(ErrorCodes), response);
            tlpBiometry.Enabled = true;
        }
        private void btnFormatFOCCard_Click(object sender, EventArgs e)
        {
            tlpBiometry.Enabled = false;
            subClearText();
            //Remove fingerprint application from card
            int response = USBReader.UninstallFingerprintFromCard();
            //Display returned Error code
            lblBiometryResponse.Text = Enum.GetName(typeof(ErrorCodes), response);
            tlpBiometry.Enabled = true;
        }
        private void btnVerify_Click(object sender, EventArgs e)
        {
            tlpBiometry.Enabled = false;
            try
            {
                subClearText();
                byte[] TemplateFromReader;
                //Get live finger template from reader
                FingerData Resault = USBReader.GetFingerTemlate();
                if (Resault.ErrorCode == (byte)ErrorCodes.OK)
                {
                    TemplateFromReader = Resault.Template;
                }
                else
                {
                    //Display returned Error code and exit
                    lblBiometryResponse.Text = Enum.GetName(typeof(ErrorCodes), Resault.ErrorCode);
                    return;
                }
                //Compare templates
                MatchResault Match = USBReader.CompareTemplates(listEntolledTemplates[Convert.ToInt32(comboFingerToVerify.Text) - 1], TemplateFromReader, Convert.ToInt32(comboSecurityLevel.Text));
                //Display returned Error code
                lblBiometryResponse.Text = Enum.GetName(typeof(MatchErrorCodes), Match.ErrorCode);
                //Display match resault
                if (Match.ErrorCode == (int)MatchErrorCodes.OK)
                {
                    if (Match.Match)
                    {
                        lblBiometryResponse.Text += " - Finger MATCH with template " + comboFingerToVerify.Text;
                    }
                }
                else
                {
                    lblBiometryResponse.Text += " - Finger DOES NOT MATCH with template " + comboFingerToVerify.Text;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            tlpBiometry.Enabled = true;
        }
        private void btnClearFingers_Click(object sender, EventArgs e)
        {
            comboFingerToVerify.Items.Clear();
            comboFingerToVerify.Enabled = false;
            comboWriteTemplateToCard.Items.Clear();
            comboWriteTemplateToCard.Enabled = false;
            listEntolledTemplates.Clear();
            lblEnrolledFingers.Text = "0";
            btnVerify.Enabled = false;
            btnFindMatch.Enabled = false;
        }
        private void comboSecurityLevel_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Set security level
            if (USBReader != null)
            {
                USBReader.SecurityLevel = Convert.ToInt32(comboSecurityLevel.Text);
            }
        }
        private void checkFastMode_CheckedChanged(object sender, EventArgs e)
        {
            //Set fast mode
            if (USBReader != null)
            {
                USBReader.FastMode = checkFastMode.Checked;
            }
        }
        private void btnFindMatch_Click(object sender, EventArgs e)
        {
            tlpBiometry.Enabled = false;
            try
            {
                subClearText();
                byte[] TemplateFromReader;
                //Get live finger template from reader
                FingerData Resault = USBReader.GetFingerTemlate();
                if (Resault.ErrorCode == (byte)ErrorCodes.OK)
                {
                    TemplateFromReader = Resault.Template;
                }
                else
                {
                    //Display returned Error code and exit
                    lblBiometryResponse.Text = Enum.GetName(typeof(ErrorCodes), Resault.ErrorCode);
                    return;
                }
                //Check for match
                FindMatchResault FindMatch = USBReader.FindMatch(TemplateFromReader, listEntolledTemplates.ToArray(), Convert.ToInt32(comboTimeout.Text));
                //Display error code
                lblBiometryResponse.Text = Enum.GetName(typeof(MatchErrorCodes), FindMatch.ErrorCode);
            //Display match resault
                if (FindMatch.ErrorCode == (int)MatchErrorCodes.OK)
                {
                    if (FindMatch.MatchIndex > -1)
                    {
                        lblBiometryResponse.Text += " - Finger MATCH with template " + (FindMatch.MatchIndex + 1).ToString();
                    }
                    else
                    {
                        lblBiometryResponse.Text += " - Finger  MATCH NOT FOUND";
                    }
                }
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        tlpBiometry.Enabled = true;
        }
        #endregion
        #region "Functions"
        private void subClearText()
        {
            lblErrorCode.Text = "";
            lblCardType.Text = "";
            lblMifareType.Text = "";
            lblID.Text = "";
            lblID_Decimal.Text = "";
            lblIDConverted.Text = "";
            lblID_Converted_Decimal.Text = "";
            lblIDSize.Text = "";
            lblReadBlockResponse.Text = "";
            lblTemplateType.Text = "";
            lblQuality.Text = "";
            lblBiometryResponse.Text = "";
        }
        private void EnableControl(int readertype)
        {
            lblDevice.Text = Enum.GetName(typeof(Devices), readertype);
            switch (readertype)
            {
                case (int)Devices.B100PROXUSB:
                    btnGetCardID.Enabled = true;
                    flpMifareRW.Enabled = true;
                    tlpBiometry.Enabled = true;
                    flpBioprox.Enabled = false;
                    break;
                case (int)Devices.BIOPROXUSB:
                    btnGetCardID.Enabled = true;
                    flpMifareRW.Enabled = true;
                    tlpBiometry.Enabled = true;
                    flpBioprox.Enabled = true;
                    break;
                case (int)Devices.PROXE:
                    btnGetCardID.Enabled = true;
                    flpMifareRW.Enabled = true;
                    tlpBiometry.Enabled = false;
                    break;
                case (int)Devices.PROXUSB:
                    btnGetCardID.Enabled = true;
                    flpMifareRW.Enabled = true;
                    tlpBiometry.Enabled = false;
                    break;
                default:
                    btnGetCardID.Enabled = false;
                    flpMifareRW.Enabled = false;
                    tlpBiometry.Enabled = false;
                    break;
            }
        }
        private decimal ByteToDecimal(byte[] bytes )
        {
            Array.Resize(ref bytes, 16);
            Int32[] bits = new Int32[4];
            for (int i = 0; i <= 15; i += 4)
            {
                bits[i / 4] = BitConverter.ToInt32(bytes, i);
            }
            return new decimal(bits);
        }
        #endregion
        #region "DLL events"
        //Reader connected/disconnected event handler
        private void USBEventHandler(object sender, DesktopReader.USBEventArgs e)
        {
            lblVersion.Text = e.version;
            EnableControl(e.device);
        }
        #endregion
    }
}
